/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus;


/**
 *  The construct for a location representation.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Location.java,v 1.1 2003/09/12 01:43:17 glchen Exp $
 */
public class Location
{
    protected String _street;
    protected String _city;
    protected String _cityurl;
    protected String _state;
    protected String _country;
    protected String _zipcode;
    protected int _certainty;


    //=======================================================================
    // Constructors.
    //=======================================================================

    public Location()
    {
    }
    
    public Location(String street, String city, 
        String state, String country, String zipcode, int certainty)
    {
        _street = street;
        _city = city;
        _state = state;
        _country = country;
        _zipcode = zipcode;
        _certainty = certainty;
    }

    //=======================================================================
    // Getters and Setters.
    //=======================================================================

    public void setStreet(String street) { _street = street; }
    public String getStreet() { return _street; }

    public void setCity(String city) { _city = city; }
    public String getCity() { return _city; }
    
    public void setCityUrl(String cityurl) { _cityurl = cityurl; }
    public String getCityUrl() { return _cityurl; }
    public String getCityHtml() 
    {
        if (_cityurl == null)
        {
            return _city;
        }
        else
        {
            return "<a href=\"" + _cityurl + "\"><font color=#ffffff>" + _city + "</font></a>";
        }
    }

    public void setState(String state) { _state = state; }
    public String getState() { return _state; }

    public void setCountry(String country) { _country = country; }
    public String getCountry() { return _country; }

    public void setZipCode(String zipcode) 
    {
        _zipcode = zipcode; 
        if (_zipcode.equals("03755"))
        {
            _cityurl = "http://www.hanovernh.org/";
        }
    }
    public String getZipCode() { return _zipcode; }

    public void setCertainty(int certainty) { _certainty = certainty; }
    public int getCertainty() { return _certainty; }

    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    public String toLogString()
    {
        String ret = "";
        ret += (_street != null) ? _street + "," : ",";
        ret += (_city != null) ? _city + "," : ",";
        ret += (_state != null) ? _state + "," : ",";
        ret += (_country != null) ? _country + "," : ",";
        ret += (_zipcode != null) ? _zipcode + "," : ",";
        ret += (_certainty != 0) ? Integer.toString(_certainty) : "";
        return ret;
    }

    public static Location parseLogString(String line)
    {
        String[] info = line.split(",");
        Location loc = new Location();
        if (info[0].length() > 0) loc.setStreet(info[0]);
        if (info[1].length() > 0) loc.setCity(info[1]);
        if (info[2].length() > 0) loc.setState(info[2]);
        if (info[3].length() > 0) loc.setCountry(info[3]);
        if (info[4].length() > 0) loc.setZipCode(info[4]);
        if (info[5].length() > 0) loc.setCertainty(Integer.parseInt(info[5]));
        return loc;
    }
    
    public String toString()
    {
        String street = (_street == null) ? "" : _street;
        String city = (_city == null) ? "" : _city;
        String state = (_state == null) ? "" : _state;
        String country = (_country == null) ? "" : _country;
        String zipcode = (_zipcode == null) ? "" : _zipcode;
        int certainty = (_certainty == 0) ? 50 : _certainty;
        return street + " " + city + " " 
            + state + " " + country + " " + zipcode 
            + " (Certainty: " + Double.toString(certainty/100.0) + ")";
    }
}
