/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package campus.compass;


import java.util.Iterator;
import java.util.Hashtable;


/**
 *  Manages the mapping of the country name to its 2-char code.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: CountryCode.java,v 1.1 2003/09/08 05:22:44 glchen Exp $
 */
public class CountryCode 
{
    private CountryCode() {}
    private static Hashtable _country;

    /** Get 2-digit code of a country for specified name. */
    public static String getCountryCode(String name)
    {
        if (_country == null)
        {
            initCountryMapping();
        }
        return (String) _country.get(name);
    }

    /** Get full name of a country for specified code. */
    public static String getCountryName(String code)
    {
        if (code == null || code.length() != 2)
        {
            throw new IllegalArgumentException("Wrong code specified.");
        }
        if (_country == null)
        {
            initCountryMapping();
        }
        String ret = null;
        Iterator itr = _country.keySet().iterator();
        while (itr.hasNext())
        {
            ret = (String) itr.next();
            if (code.equals(_country.get(ret)))
            {
                break;
            }
        }
        return ret;
    }
    
    private synchronized static void initCountryMapping()
    {
        _country = new Hashtable();
        _country.put("Afghanistan", "AF");
        _country.put("Albania", "AL");
        _country.put("Algeria", "DZ");
        _country.put("American Samoa", "AS");
        _country.put("Andorra", "AD");
        _country.put("Angola", "AO");
        _country.put("Anguilla", "AI");
        _country.put("Antarctica", "AQ");
        _country.put("Antigua and Barbuda", "AG");
        _country.put("Argentina", "AR");
        _country.put("Armenia", "AM");
        _country.put("Aruba", "AW");
        _country.put("Australia", "AU");
        _country.put("Austria", "AT");
        _country.put("Azerbaijan", "AZ");
        _country.put("Bahamas", "BS");
        _country.put("Bahrain", "BH");
        _country.put("Bangladesh", "BD");
        _country.put("Barbados", "BB");
        _country.put("Belarus", "BY");
        _country.put("Belgium", "BE");
        _country.put("Belize", "BZ");
        _country.put("Benin", "BJ");
        _country.put("Bermuda", "BM");
        _country.put("Bhutan", "BT");
        _country.put("Bolivia", "BO");
        _country.put("Bosnia and Herzegovina", "BA");
        _country.put("Botswana", "BW");
        _country.put("Bouvet Island", "BV");
        _country.put("Brazil", "BR");
        _country.put("British Indian Ocean Territory", "IO");
        _country.put("Brunei Darussalam", "BN");
        _country.put("Bulgaria", "BG");
        _country.put("Burkina Faso", "BF");
        _country.put("Burundi", "BI");
        _country.put("Cambodia", "KH");
        _country.put("Cameroon", "CM");
        _country.put("Canada", "CA");
        _country.put("Cape Verde", "CV");
        _country.put("Cayman Islands", "KY");
        _country.put("Central African Republic", "CF");
        _country.put("Chad", "TD");
        _country.put("Chile", "CL");
        _country.put("China", "CN");
        _country.put("Christmas Island", "CX");
        _country.put("Cocos", "CC");
        _country.put("Colombia", "CO");
        _country.put("Comoros", "KM");
        _country.put("Congo", "CG");
        _country.put("Cook Islands", "CK");
        _country.put("Costa Rica", "CR");
        _country.put("Cote D'Ivoire", "CI");
        _country.put("Croatia (Hrvatska", "HR");
        _country.put("Cuba", "CU");
        _country.put("Cyprus", "CY");
        _country.put("Czech Republic", "CZ");
        _country.put("Denmark", "DK");
        _country.put("Djibouti", "DJ");
        _country.put("Dominica", "DM");
        _country.put("Dominican Republic", "DO");
        _country.put("East Timor", "TP");
        _country.put("Ecuador", "EC");
        _country.put("Egypt", "EG");
        _country.put("El Salvador", "SV");
        _country.put("Equatorial Guinea", "GQ");
        _country.put("Eritrea", "ER");
        _country.put("Estonia", "EE");
        _country.put("Ethiopia", "ET");
        _country.put("Falkland Islands", "FK");
        _country.put("Faroe Islands", "FO");
        _country.put("Fiji", "FJ");
        _country.put("Finland", "FI");
        _country.put("France", "FR");
        _country.put("France, Metropolitan", "FX");
        _country.put("French Guiana", "GF");
        _country.put("French Polynesia", "PF");
        _country.put("French Southern Territories", "TF");
        _country.put("Gabon", "GA");
        _country.put("Gambia", "GM");
        _country.put("Georgia", "GE");
        _country.put("Germany", "DE");
        _country.put("Ghana", "GH");
        _country.put("Gibraltar", "GI");
        _country.put("Greece", "GR");
        _country.put("Greenland", "GL");
        _country.put("Grenada", "GD");
        _country.put("Guadeloupe", "GP");
        _country.put("Guam", "GU");
        _country.put("Guatemala", "GT");
        _country.put("Guinea", "GN");
        _country.put("Guinea-Bissau", "GW");
        _country.put("Guyana", "GY");
        _country.put("Haiti", "HT");
        _country.put("Heard and McDonald Islands", "HM");
        _country.put("Honduras", "HN");
        _country.put("Hong Kong", "HK");
        _country.put("Hungary", "HU");
        _country.put("Iceland", "IS");
        _country.put("India", "IN");
        _country.put("Indonesia", "ID");
        _country.put("Iran", "IR");
        _country.put("Iraq", "IQ");
        _country.put("Ireland", "IE");
        _country.put("Israel", "IL");
        _country.put("Italy", "IT");
        _country.put("Jamaica", "JM");
        _country.put("Japan", "JP");
        _country.put("Jordan", "JO");
        _country.put("Kazakhstan", "KZ");
        _country.put("Kenya", "KE");
        _country.put("Kiribati", "KI");
        _country.put("Korea", "KP");
        _country.put("Korea", "KR");
        _country.put("Kuwait", "KW");
        _country.put("Kyrgyzstan", "KG");
        _country.put("Laos", "LA");
        _country.put("Latvia", "LV");
        _country.put("Lebanon", "LB");
        _country.put("Lesotho", "LS");
        _country.put("Liberia", "LR");
        _country.put("Libya", "LY");
        _country.put("Liechtenstein", "LI");
        _country.put("Lithuania", "LT");
        _country.put("Luxembourg", "LU");
        _country.put("Macau", "MO");
        _country.put("Macedonia", "MK");
        _country.put("Madagascar", "MG");
        _country.put("Malawi", "MW");
        _country.put("Malaysia", "MY");
        _country.put("Maldives", "MV");
        _country.put("Mali", "ML");
        _country.put("Malta", "MT");
        _country.put("Marshall Islands", "MH");
        _country.put("Martinique", "MQ");
        _country.put("Mauritania", "MR");
        _country.put("Mauritius", "MU");
        _country.put("Mayotte", "YT");
        _country.put("Mexico", "MX");
        _country.put("Micronesia", "FM");
        _country.put("Moldova", "MD");
        _country.put("Monaco", "MC");
        _country.put("Mongolia", "MN");
        _country.put("Montserrat", "MS");
        _country.put("Morocco", "MA");
        _country.put("Mozambique", "MZ");
        _country.put("Myanmar", "MM");
        _country.put("Namibia", "NA");
        _country.put("Nauru", "NR");
        _country.put("Nepal", "NP");
        _country.put("Netherlands", "NL");
        _country.put("Netherlands Antilles", "AN");
        _country.put("New Caledonia", "NC");
        _country.put("New Zealand", "NZ");
        _country.put("Nicaragua", "NI");
        _country.put("Niger", "NE");
        _country.put("Nigeria", "NG");
        _country.put("Niue", "NU");
        _country.put("Norfolk Island", "NF");
        _country.put("Northern Mariana Islands", "MP");
        _country.put("Norway", "NO");
        _country.put("Oman", "OM");
        _country.put("Pakistan", "PK");
        _country.put("Palau", "PW");
        _country.put("Panama", "PA");
        _country.put("Papua New Guinea", "PG");
        _country.put("Paraguay", "PY");
        _country.put("Peru", "PE");
        _country.put("Philippines", "PH");
        _country.put("Pitcairn", "PN");
        _country.put("Poland", "PL");
        _country.put("Portugal", "PT");
        _country.put("Puerto Rico", "PR");
        _country.put("Qatar", "QA");
        _country.put("Reunion", "RE");
        _country.put("Romania", "RO");
        _country.put("Russian Federation", "RU");
        _country.put("Rwanda", "RW");
        _country.put("Saint Kitts and Nevis", "KN");
        _country.put("Saint Lucia", "LC");
        _country.put("Saint Vincent and The Grenadines", "VC");
        _country.put("Samoa", "WS");
        _country.put("San Marino", "SM");
        _country.put("Sao Tome and Principe", "ST");
        _country.put("Saudi Arabia", "SA");
        _country.put("Senegal", "SN");
        _country.put("Seychelles", "SC");
        _country.put("Sierra Leone", "SL");
        _country.put("Singapore", "SG");
        _country.put("Slovak Republic", "SK");
        _country.put("Slovenia", "SI");
        _country.put("Solomon Islands", "SB");
        _country.put("Somalia", "SO");
        _country.put("South Africa", "ZA");
        _country.put("S. Georgia and S. Sandwich Isls.", "GS");
        _country.put("Spain", "ES");
        _country.put("Sri Lanka", "LK");
        _country.put("St. Helena", "SH");
        _country.put("St. Pierre and Miquelon", "PM");
        _country.put("Sudan", "SD");
        _country.put("Suriname", "SR");
        _country.put("Svalbard and Jan Mayen Islands", "SJ");
        _country.put("Swaziland", "SZ");
        _country.put("Sweden", "SE");
        _country.put("Switzerland", "CH");
        _country.put("Syria", "SY");
        _country.put("Taiwan", "TW");
        _country.put("Tajikistan", "TJ");
        _country.put("Tanzania", "TZ");
        _country.put("Thailand", "TH");
        _country.put("Togo", "TG");
        _country.put("Tokelau", "TK");
        _country.put("Tonga", "TO");
        _country.put("Trinidad and Tobago", "TT");
        _country.put("Tunisia", "TN");
        _country.put("Turkey", "TR");
        _country.put("Turkmenistan", "TM");
        _country.put("Turks and Caicos Islands", "TC");
        _country.put("Tuvalu", "TV");
        _country.put("Uganda", "UG");
        _country.put("Ukraine", "UA");
        _country.put("United Arab Emirates", "AE");
        _country.put("United Kingdom", "UK");
        _country.put("United States", "US");
        _country.put("US Minor Outlying Islands", "UM");
        _country.put("Uruguay", "UY");
        _country.put("Uzbekistan", "UZ");
        _country.put("Vanuatu", "VU");
        _country.put("Vatican City State", "VA");
        _country.put("Venezuela", "VE");
        _country.put("Viet Nam", "VN");
        _country.put("Virgin Islands", "VG");
        _country.put("Virgin Islands", "VI");
        _country.put("Wallis and Futuna Islands", "WF");
        _country.put("Western Sahara", "EH");
        _country.put("Yemen", "YE");
        _country.put("Yugoslavia", "YU");
        _country.put("Zaire", "ZR");
        _country.put("Zambia", "ZM");
        _country.put("Zimbabwe", "ZW");
    }
}
