/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.snmp;


import serp.util.Options;
import org.opennms.protocols.snmp.*;
import java.net.InetAddress;


/**
 *  A SNMP test using joesnmp library. It seems it lacks a mib parser and we
 *  have to supply ASN1 formated string as object ID for query, not very
 *  convenient.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SnmpTest.java,v 1.1 2003/09/04 16:40:02 glchen Exp $
 */
public class SnmpTest 
    implements SnmpHandler
{
	String		m_startOid	= ".1.3.6.1.2.1.1.3";
	SnmpObjectId	m_stopAt	= null;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public SnmpTest()
    {
    }


    //=======================================================================
    // Implementation of org.opennms.protocols.snmp.SnmpHandler interface.
    //=======================================================================

	public void snmpInternalError(SnmpSession session, int err, SnmpSyntax pdu)
	{
		System.err.println("An unexpected error occured with the SNMP Session");
		System.err.println("The error code is " + err);
		synchronized(session)
		{
			session.notify();
		}
	}
    
	public void snmpTimeoutError(SnmpSession session, SnmpSyntax pdu)
	{
		System.err.println("The session timed out trying to communicate with the remote host");
		synchronized(session)
		{
			session.notify();
		}
	}
	
	public void snmpReceivedPdu(SnmpSession session, int cmd, SnmpPduPacket pdu)
	{
		SnmpPduRequest req = null;
		if(pdu instanceof SnmpPduRequest)
		{
			req = (SnmpPduRequest)pdu;
		}
		
		if(pdu.getCommand() != SnmpPduPacket.RESPONSE)
		{
			System.err.println("Error: Received non-response command " + pdu.getCommand());
			synchronized(session)
			{
				session.notify();
			}
			return;
		}
		
		if(req.getErrorStatus() != 0)
		{
			System.out.println("End of mib reached");
			synchronized(session)
			{
				session.notify();
			}
			return;
		}
		
		//
		// Passed the checks so lets get the first varbind and
		// print out it's value
		//
		SnmpVarBind vb = pdu.getVarBindAt(0);
		if(vb.getValue().typeId() == SnmpEndOfMibView.ASNTYPE ||
		   (m_stopAt != null && m_stopAt.compare(vb.getName()) < 0))
		{
			System.out.println("End of mib reached");
			synchronized(session)
			{
				session.notify();
			}
			return;
		}

		System.out.println(vb.getName().toString() + ": " + vb.getValue());

		//
		// make the next pdu
		//
		SnmpVarBind[] vblist  = { new SnmpVarBind(vb.getName()) };
		SnmpPduRequest newReq = new SnmpPduRequest(SnmpPduPacket.GETNEXT, vblist);
		newReq.setRequestId(SnmpPduPacket.nextSequence());
		
		session.send(newReq);
	}


    //=======================================================================
    // Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);
        
		SnmpTest walker = new SnmpTest();
		InetAddress remote = null;
        remote = InetAddress.getByName(
            opts.getProperty("ap", "sudi11-ap"));
        walker.m_startOid = opts.getProperty("oid", walker.m_startOid);
        
		//
		// Initialize the peer
		//
		SnmpPeer peer = new SnmpPeer(remote);
        //peer.setPort(walker.m_port);
        //peer.setTimeout(walker.m_timeout);
        //peer.setRetries(walker.m_retries);

		SnmpParameters parms = peer.getParameters();
		parms.setVersion(SnmpSMI.SNMPV2);
        parms.setReadCommunity(opts.getProperty("community", "invalid"));

		//
		// Now create the session, set the initial request
		// and walk the tree!
		//
		SnmpSession session = null;
        session = new SnmpSession(peer);
		session.setDefaultHandler(walker);

		//
		// set the stop point
		//
		SnmpObjectId id = new SnmpObjectId(walker.m_startOid);
		int[] ids = id.getIdentifiers();
		++ids[ids.length-1];
		id.setIdentifiers(ids);
		walker.m_stopAt = id;
        
		//
		// send the first request
		//
		SnmpVarBind[]  vblist = { new SnmpVarBind(walker.m_startOid) };
		SnmpPduRequest pdu = new SnmpPduRequest(SnmpPduPacket.GETNEXT, vblist);
		pdu.setRequestId(SnmpPduPacket.nextSequence());
		try
		{
			synchronized(session)
			{
				session.send(pdu);
				session.wait();
			}
		}
		catch(InterruptedException e) 
		{ 
			// do nothing
		}
		finally
		{
			session.close();
		}
    }
}
