/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package org.doit.muffin.filter;


import org.doit.io.*;
import org.doit.html.*;
import org.doit.muffin.*;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Hashtable;

import solar.util.NetUtil;
import solar.util.SystemUtil;
import campus.Location;
import campus.Building;
import campus.compass.CompassService;
import campus.snmp.ApStat;


/**
 *  A compass filter impose location-dependent information on top of
 *  the replied page to client.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: CompassFilter.java,v 1.8 2003/09/13 03:52:42 glchen Exp $
 */
public class CompassFilter 
    implements ContentFilter
{
    public static final String COMPASS_COLOR = "#663333"; //"#3366cc";

    protected Prefs prefs;
    protected Reply reply;
    protected Request request;
    protected InputObjectStream in = null;
    protected OutputObjectStream out = null;

    protected String _clientAp = null;    // for wireless client
    protected Location _clientLocation = null;


    //=======================================================================
    // Constructors.
    //=======================================================================

    public CompassFilter()
    {
    }

    
    //=======================================================================
    // Implementation of ContentFilter interface.
    //=======================================================================

    public void setPrefs(Prefs prefs)
    {
        this.prefs = prefs;
    }
    
    public boolean needsFiltration(Request request, Reply reply)
    {
        this.request = request;
        this.reply = reply;
	
        String s = reply.getContentType();
        return s != null && s.startsWith("text/html");
    }
    
    public void setInputObjectStream(InputObjectStream in)
    {
    	this.in = in;
    }

    public void setOutputObjectStream(OutputObjectStream out)
    {
        this.out = out;
    }
    
    public void run()
    {
        Thread.currentThread().setName("CompassFilter");
        try
        {
            getClientLocation();
        }
        catch (Exception ex) {}
        try
        {
            boolean found = false;
            Object obj;
            while ((obj = in.read()) != null)
            {
                Token token = (Token) obj;
                if (token.getType() == Token.TT_TAG)
                {
                    Tag tag = token.createTag();
                    if (tag.is("/title"))
                    {
                        out.write(obj);
                        out.write(getStyleHtml());
                    }
                    else if (tag.is("body"))
                    {
                        out.write(obj);
                        if (!found)
                        {
                            out.write(getCompassHtml());
                            found = true;
                        }
                    }
                    else
                    {
                        out.write(obj);
                    }
                }
                else
                {
                    out.write(obj);
                }
            }
            if (!found) /* no body or /body found */
            {
                out.write(getCompassHtml());
            }
        }
        catch (Exception ioe)
        {
            ioe.printStackTrace();
        }
        finally
        {
            try
            {
                out.flush();
                out.close();
            }
            catch (IOException ioe) {}
        }
    }

    private void getClientLocation()
        throws Exception
    {
        String ip = getClientIp();
        _clientAp = CompassService.getApByIp(ip);
        _clientLocation = CompassService.getLocationByIp(ip);
    }

    private String getClientIp()
        throws Exception
    {
        String ret = NetUtil.getIpAddress(request.getClient().getInetAddress());
        if (ret.equals("127.0.0.1"))
        {
            ret = NetUtil.getIpAddress(InetAddress.getLocalHost());
        }
        return ret;
    }

    private Token getStyleHtml()
    {
        Token ret = new Token();
        ret.append("<style>\n");
        ret.append("<!--\n");
        ret.append("body,td,div,.p,a{font-family:arial,sans-serif }\n");
        ret.append("div,td{color:#000}\n");
        ret.append(".f,.fl:link{color:#6f6f6f}\n");
        ret.append("a:link,.w,a.w:link,.w a:link{color:#00c}\n");
        ret.append("a:visited,.fl:visited{color:#551a8b}\n");
        ret.append("a:active,.fl:active{color:#f00}\n");
        ret.append(".t a:link,.t a:active,.t a:visited,.t{color:#ffffff}\n");
        ret.append(".t{background-color:" + COMPASS_COLOR + "}\n");
        ret.append(".h{color:" + COMPASS_COLOR + "}\n");
        ret.append(".i,.i:link{color:#a90a08}\n");
        ret.append(".a,.a:link{color:#008000}\n");
        ret.append(".z{display:none}\n");
        ret.append("div.n {margin-top: 1ex}\n");
        ret.append(".n a{font-size:10pt; color:#000}\n");
        ret.append(".n .i{font-size:10pt; font-weight:bold}\n");
        ret.append(".q a:visited,.q a:link,.q a:active,.q {text-decoration: none; color: #00c;}\n");
        ret.append(".b{font-size: 12pt; color:#00c; font-weight:bold}\n");
        ret.append(".ch{cursor:pointer;cursor:hand}\n");
        ret.append(".e{margin-top: .75em; margin-bottom: .75em}\n");
        ret.append(".g{margin-top: 1em; margin-bottom: 1em}\n");
        ret.append("form{margin-bottom:0; margin-top: 0}");
        ret.append("//-->\n");
        ret.append("</style>\n");
        return ret;
    }

    private Token getCompassHtml()
    {
        Token ret = new Token();
        ret.append(getCompassHeader());
        if (_clientLocation != null
            && _clientLocation.getZipCode() != null
            && _clientLocation.getZipCode().equals("03755"))
        {
            ret.append(getCompassTab(
                "News", "http://www.thedartmouth.com"));
            ret.append(getCompassTab(
                "Dartmouth", "http://www.dartmouth.edu/home/explore/"));
            ret.append(getCompassTab(
                "Events", "http://www.vnews.com/calendar/calendar.htm"));
            ret.append(getCompassTab(
                "Classified", "http://www.itsclassified.com/"));
        }
        ret.append(getCompassTab("Weather", 
            appendCompassTag(getWeatherUrl())));
        ret.append(getCompassTab("TV", 
            appendCompassTag(getTvUrl())));
        ret.append(getCompassTab("Others", 
            appendCompassTag(getYahooUrl())));
        ret.append(getCompassBody());
        ret.append(getCompassFooter());
        ret.append("</table></td></tr></table>\n");
        return ret;
    }

    private String getCompassHeader()
    {
        StringBuffer ret = new StringBuffer();
        ret.append("<table border=0 cellpadding=0 cellspacing=0><tr>\n");
        ret.append("<td id=t0 bgcolor=" + COMPASS_COLOR + " width=60 nowrap><center>\n");
        ret.append("&nbsp;<font size=-1 color=#ffffff><b>Local</b></font>&nbsp;</center></td>\n");
        ret.append("<td width=1 bgcolor=#808080><img width=1 height=1 alt=\"\"></td>\n");
        ret.append("<td width=1 bgcolor=#ffffff><img width=1 height=1 alt=\"\"></td>\n");
        return ret.toString();
    }

    private String getCompassTab(String name, String url)
    {
        StringBuffer ret = new StringBuffer();
        ret.append("<td id=t1 style=cursor:pointer;cursor:hand; class=q ");
        ret.append("bgcolor=#efefef width=60 nowrap><center><font size=-1><a href=\"");
        ret.append(url);
        ret.append("\" class=\"q\">&nbsp;");
        ret.append(name);
        ret.append("&nbsp;</a></font></center></td><td width=1 bgcolor=#808080>");
        ret.append("<img width=1 height=1 alt=\"\"></td>");
        return ret.toString();
    }

    private String getCompassBody()
    {
        String loc = (_clientLocation == null) ? 
            "Unable to determine location" : getLocationHtml(_clientLocation);

        StringBuffer ret = new StringBuffer();
        ret.append("</tr></table>\n");
        ret.append("<table width=100% border=0 cellpadding=1 cellspacing=0 bgcolor=");
        ret.append(COMPASS_COLOR + "><tr>\n");
        ret.append("<td bgcolor=" + COMPASS_COLOR + " nowrap>\n");
        ret.append("<table width=100% border=0 cellpadding=1 cellspacing=0 bgcolor=");
        ret.append(COMPASS_COLOR + "><tr>\n");
        ret.append(getGoogleTab());
        ret.append("<td align=right bgcolor=" + COMPASS_COLOR + " nowrap>&nbsp;\n");
        ret.append("<font size=-1 color=#ffffff>" + loc + ".</font></a></td>\n");
//        ret.append("<td align=right><font size=-1 color=#ffffff>");
//        ret.append("V" + CompassService.VERSION + "</font></td>\n");
        ret.append("</tr>");
        return ret.toString();
    }

    private String getCompassFooter()
    {
        try
        {
            ApStat apstat = CompassService.getApStatByIp(getClientIp());
            if (apstat == null)
            {
                return "";
            }
            StringBuffer ret = new StringBuffer();
            ret.append("<tr><td bgcolor=ffffff nowrap colspan=2><font size=-1>");
            ret.append("&nbsp;Info: You are using access point [");
            ret.append(apstat.getApName() + "], which has been up for [");
            ret.append(apstat.getUpTime() + "] and there are [");
            ret.append(Integer.toString(apstat.getNumClients()) + "] clients associated.");
            ret.append("</font></td></tr>");
            return ret.toString();
        }
        catch (Exception ex)
        {
            return "";
        }
    }

    private String getLocationHtml(Location loc)
    {
        StringBuffer sb = new StringBuffer();
//        sb.append("<b>Compass</b>: ");
        if (loc instanceof Building)
        {
            Building bd = (Building) loc;
            if (bd.getName() != null)   // use building name
            {
                sb.append(bd.getNameHtml() + " ");
            }
            else if (bd.getStreet() != null)    //  use street address
            {
                sb.append(bd.getStreet() + " ");
            }
            else if (_clientAp != null) // use associated AP stripped name
            {
                sb.append(_clientAp + " ");
            }
        }
        sb.append(loc.getCityHtml() + " ");
        sb.append(loc.getState() + " ");
        sb.append(loc.getCountry() + " ");
        sb.append(loc.getZipCode() + " ");
        sb.append("(Certainty: " + Double.toString(loc.getCertainty()/100.0) + ")");
        return sb.toString();
    }

    private String getGoogleTab()
    {
        String loc = "";
        if (_clientLocation != null)
        {
            loc = _clientLocation.getCity() + " " + _clientLocation.getState();
        }
        StringBuffer ret = new StringBuffer();
        ret.append("<td align=left valign=top>");
        ret.append("<form action=\"http://www.google.com/search\">");
        ret.append("<input style=\"font-size: 9pt\" maxLength=256 size=30 name=q value=\"" + loc + "\">");
        ret.append("&nbsp;<input style=\"font-size: 8pt\" type=submit value=\"Google\">");
        ret.append("</form></td>");
        return ret.toString();
    }

    private String getWeatherUrl()
    {
        if (_clientLocation != null 
            && _clientLocation.getZipCode() != null)
        {
            return "http://weather.yahoo.com/search/weather2?p=" 
                + _clientLocation.getZipCode();
        }
        else
        {
            return "http://weather.yahoo.com/";
        }
    }

    private String getTvUrl()
    {
        if (_clientLocation != null 
            && _clientLocation.getZipCode() != null)
        {
            return "http://tv.yahoo.com/lineup?zip=" 
                + _clientLocation.getZipCode();
        }
        else
        {
            return "http://tv.yahoo.com/lineup";
        }
    }

    private String getYahooUrl()
    {
        if (_clientLocation != null 
            && _clientLocation.getZipCode() != null)
        {
            return "http://local.yahoo.com/golocation?csz=" 
                + _clientLocation.getZipCode();
        }
        else
        {
            return "http://local.yahoo.com/";
        }
    }

    private String appendCompassTag(String url)
    {
        if (url.indexOf("?") != -1)
        {
            return url + "&compass=true";
        }
        else
        {
            return url + "?compass=true";
        }
    }
}
