/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package rice.pastry.socket;

import rice.pastry.*;
import rice.pastry.dist.*;
import rice.pastry.routing.*;
import rice.pastry.messaging.*;

import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.*;
import java.util.*;
import java.net.*;


/**
 *  A direct mimic to WireNodeHandlePool class.
 *  The WireNodeHandlePool controls all of the node handles in
 *  use by the WirePastryNode.  It ensures that there is only one
 *  node handle for each respective pastry node.
 *
 *  @author  Guanling Chen
 *  @author  Alan Mislove
 *  @version $Id: SocketNodeHandlePool.java,v 1.2 2003/09/22 19:53:06 glchen Exp $
 */
public class SocketNodeHandlePool 
    extends DistNodeHandlePool 
{
    private HashMap handles;
    private SocketPastryNode pastryNode;

    public SocketNodeHandlePool(SocketPastryNode spn) 
    {
        super();
        pastryNode = spn;
        handles = new HashMap();
    }


    public synchronized DistNodeHandle coalesce(DistNodeHandle handle) 
    {
        if ((handles.get(handle.getNodeId()) == null) 
            || (handles.get(handle.getNodeId()) == handle)) 
        {
            handles.put(handle.getNodeId(), handle);
            handle.setIsInPool(true);
        } 
        else 
        {
            handle.setIsInPool(false);
        }
        DistNodeHandle response = (DistNodeHandle) handles.get(handle.getNodeId());
        return response;
    }

    public SocketNodeHandle get(NodeId nodeId) 
    {
        return (SocketNodeHandle) handles.get(nodeId);
    }

    private void debug(String s) 
    {
        if (Log.ifp(6))
            System.out.println(pastryNode.getNodeId() + " (P): " + s);
    }

    public String toString() 
    {
        String response = "";
        Iterator i = handles.keySet().iterator();
        while (i.hasNext()) 
        {
            Object o = i.next();
            response += o + "\t->\t" + handles.get(o) + "\n";
        }
        return response;
    }
}
