/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package rice.pastry.socket;


import java.util.*;
import java.io.*;
import java.net.*;

import rice.pastry.*;
import rice.pastry.dist.*;
import rice.pastry.leafset.*;
import rice.pastry.routing.*;
import rice.pastry.messaging.*;
import rice.pastry.join.*;
import rice.pastry.client.*;

import solar.service.transport.*;


/**
 *  A direct mimic of WirePastryNode class.
 *  An Socket-based Pastry node, which has two threads - one thread
 *  for performing route set and leaf set maintainance, and another
 *  thread for listening on the sockets and performing all non-blocking
 *  I/O.
 *
 *  @author  Guanling Chen
 *  @author  Alan Mislove
 *  @version $Id: SocketPastryNode.java,v 1.2 2003/09/22 19:53:06 glchen Exp $
 */

public class SocketPastryNode 
    extends DistPastryNode 
{
    protected Transport _transport;
    
    // The address (ip + port) of this pastry node
    private InetSocketAddress _address;

    // The pool of all node handles
    private SocketNodeHandlePool _pool;
  
    // The thread in which the SelectorManager is running
    private Thread _executionThread;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public SocketPastryNode(NodeId id) 
    {
        super(id);
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void setSocketElements(
        InetSocketAddress address,
        Transport transport,
        SocketNodeHandlePool pool,
        int lsmf,
        int rsmf)
    {
        _address = address;
        _transport = transport;
        _pool = pool;
        leafSetMaintFreq = lsmf;
        routeSetMaintFreq = rsmf;
    }

    public Transport getTransport()
    {
        return _transport;
    }

    public void setThread(Thread t) 
    {
        _executionThread = t;
    }
  
    public boolean inThread() 
    {
        return _executionThread.equals(Thread.currentThread());
    }

    public void doneNode(NodeHandle bootstrap)
    {
        super.doneNode(bootstrap);
        initiateJoin(bootstrap);
    }

    public DistNodeHandlePool getNodeHandlePool() 
    {
        return _pool;
    }

    public void nodeIsReady() 
    {
        super.nodeIsReady();
    }

    public void kill() 
    {
        super.kill();
    }
}
