/*************************************************************************

"FreePastry" Peer-to-Peer Application Development Substrate

Copyright 2002, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

********************************************************************************/

package rice.pastry.wire;


import java.io.*;
import java.util.*;
import java.net.*;
import rice.pastry.*;
import rice.pastry.messaging.*;
import rice.pastry.wire.messaging.socket.*;


/**
 *  Node handle that use UDP to transmit messages.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: UdpNodeHandle.java,v 1.2 2003/09/20 21:35:53 glchen Exp $
 */
public class UdpNodeHandle 
    extends WireNodeHandle
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public UdpNodeHandle(InetSocketAddress address, NodeId nid)
    {
        super(address, nid);
    }


    /**
     *  Called to send a message to the node corresponding to this handle.
     *
     *  @param msg Message to be delivered, may or may not be routeMessage.
     */
    public void receiveMessageImpl(Message msg) 
    {
        assertLocalNode();
        WirePastryNode spn = (WirePastryNode) getLocalNode();
        if (isLocal) 
        {
            spn.receiveMessage(msg);
            return;
        } 

        try {
            // if message is small enough, send via UDP
            if (messageSize(msg) <= MAX_UDP_MESSAGE_SIZE) 
            {
                ((WirePastryNode) 
                    getLocalNode()).getDatagramManager().write(
                        nodeId, address, msg);
            } 
            else 
            {
                throw new IllegalArgumentException("Message is too large"
                    + " for the UDP transport.");
            }
        } 
        catch (IOException e) 
        {
            System.out.println("IOException " + e 
                + " serializing message " + msg + " - cancelling message.");
        }
    }
}
