/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar;


import serp.util.Options;
import org.apache.log4j.*;
import java.io.*;
import java.util.*;

import solar.api.Id;
import solar.api.Guid;
import solar.api.Event;
import solar.impl.PlainEvent;
import solar.service.SolarServices;
import solar.service.ServiceManager;
import solar.service.dispatch.DispatchService;
import solar.service.directory.Name;
import solar.service.directory.PlainName;
import solar.service.directory.NameRecord;
import solar.service.directory.Advertisement;
import solar.service.directory.DirectoryListener;
import solar.service.directory.DirectoryService;
import solar.service.multicast.MulticastMember;
import solar.service.multicast.MulticastService;
import solar.service.rmtobj.RemoteObjectService;
import solar.service.transport.SocketAddress;
import solar.service.transport.PastryAddress;


/**
 *  A service platform for Solar, which contains an overlay of Planets.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Planet.java,v 1.34 2003/10/08 13:18:55 glchen Exp $
 */
public class Planet
    implements MulticastMember, DirectoryListener
{
    private static Logger _log = Logger.getLogger("solar");

    
    //=======================================================================
    //  Constructor.
    //=======================================================================

    public Planet()
    {
    }
    

    //=======================================================================
    //  Implementation of solar.service.directory.DirectoryListener interface.
    //=======================================================================

    public void handleMulticastEvent(Event evt)
    {
        System.out.println(evt);
    }

    public void nameAdded(Name name, NameRecord rec) 
    {
        System.out.println("Directory: new name " + name + " added.");
    }

    public void nameRemoved(Name name, NameRecord rec) 
    {
    }

    public void nameUpdated(Name oldName, 
        NameRecord oldrec, Name newName, NameRecord newrec) 
    {
    }

    public void nameMigrated(Name name, NameRecord oldrec, NameRecord newrec) 
    {
    }
    
    //=======================================================================
    //  Public interface methods.
    //=======================================================================

    public void run(Options opts)
        throws Exception
    {
        ServiceManager sm = SolarServices.getInstance();
        sm.init(opts);

        // initiate dispatch service first
        DispatchService ds = (DispatchService) sm.getService("dispatch");
        // directory test
        DirectoryService ns = (DirectoryService) sm.getService("directory");
        
/*        ns.advertise(new PlainName("[a=1, b=2]"), 
            Guid.makeGuid(ds.getTransportAddress("core_transport")), null);
        ns.addListener(new PlainName("[source=clock]"), this);

        Iterator itr = ns.lookup(new PlainName("[b=2]")).iterator();
        while (itr.hasNext())
        {
            Advertisement adv = (Advertisement) itr.next();
            NameRecord rec = adv.getNameRecord();
            System.out.println(
                adv.getNameRecord().getAddress() 
                + " - " + adv.getName());
        }

//        RemoteObjectService rmtobj = (RemoteObjectService) sm.getService("rmtobj");
//        rmtobj.register(new Id("planet"));


/*        MulticastService ms = (MulticastService) sm.getService("multicast");
        // multicast test
        boolean send = opts.getBooleanProperty("send");
        if (send)
        {
            ms.create("glchen-group");
            BufferedReader in = 
                new BufferedReader(new InputStreamReader(System.in));
            String line = in.readLine();
            while (true)
            {
//                PastryAddress dest = new PastryAddress();   // random destination
//                tpt.send(new StringAdu(line), dest);
                try
                {
                    Event evt = new PlainEvent(line);
                    ms.multicast("glchen-group", evt);
//                    String dest = (String) evt.getValue("dest");
//                    dest = (dest == null) ? "hogback:9988" : dest;
//                    SocketAddress daddr = new SocketAddress(dest);
//                    tpt.dispatch(evt, daddr);
                    line = in.readLine();
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();
                    line = in.readLine();
                }
            }
        }
        else
        {
            ms.join("glchen-group", this);
            //while (true)
            //{
                //System.out.println(tpt.receive());
            //}
        }
*/
    }


    //=======================================================================
    //  Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        configLogger(_log, opts);

        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
        {
            opts.load(new FileInputStream(cfg));
            opts.setFromCmdLine(args);  // cmd opts overwrite opts in file
        }

        try
        {
            new Planet().run(opts);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    public static void configLogger(Logger log, Options opts)
    {
        // set the logging verbose level
        boolean debug = opts.getBooleanProperty("debug", false);
        if (debug) {
            log.setLevel( (Level) Level.DEBUG );
        }
        else {
            log.setLevel( (Level) Level.INFO );
        }
        Layout layout = null;
        String lt = opts.getProperty("layout");
        if (lt == null) {
            layout = new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN);
        }
        else if (lt.equals("simple")) {
            layout = new SimpleLayout();
        }
        else {
            layout = new PatternLayout(lt);
        }
        ConsoleAppender appender = new ConsoleAppender(layout);
        BasicConfigurator.configure(appender);
    }
}
