/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.impl;


import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import solar.api.Attribute;
import solar.api.AttributeRecord;
import solar.SolarConstants;


/**
 *  An abstract implementation of attribute record.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: AbstractAttributeRecord.java,v 1.6 2003/08/30 17:01:47 glchen Exp $
 */
public abstract class AbstractAttributeRecord 
    implements AttributeRecord, SolarConstants
{
    private static final long serialVersionUID = ATTRIBUTE_RECORD_SVID;
    
    protected List _attributes;


    //=======================================================================
    // Implementation of solar.api.AttributeRecord interface.
    //=======================================================================
    
    public Attribute getAttribute(String key)
    {
        Attribute ret = null;
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            // return the first attribute that has the specified key
            Attribute attr = (Attribute) itr.next();
            if (attr.getKey().equals(key))
            {
                ret = attr;
                break;
            }
        }
        return ret;
    }
    
    public List getAttributes(String key)
    {
        List ret = new Vector();
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            // return all attributes that has the specified key
            Attribute attr = (Attribute) itr.next();
            if (attr.getKey().equals(key))
                ret.add(attr);
        }
        return ret;
    }

    public List getAttributes()
    {
        return _attributes;
    }

    public List getKeys()
    {
        List ret = new Vector();
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            Attribute attr = (Attribute) itr.next();
            ret.add(attr.getKey());
        }
        return ret;
    }

    public List getValues()
    {
        List ret = new Vector();
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            Attribute attr = (Attribute) itr.next();
            ret.add(attr.getValue());
        }
        return ret;
    }
    
    public Object getValue(String key)
    {
        Object ret = null;
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            // return value of first attribute that has the specified key
            Attribute attr = (Attribute) itr.next();
            if (attr.getKey().equals(key))
            {
                ret = attr.getValue();
                break;
            }
        }
        return ret;
    }

    public List getValues(String key)
    {
        List ret = new Vector();
        Iterator itr = _attributes.iterator();
        while (itr.hasNext())
        {
            // return values of all attributes that has the specified key
            Attribute attr = (Attribute) itr.next();
            if (attr.getKey().equals(key))
                ret.add(attr.getValue());
        }
        return ret;
    }
}
