/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.impl;


import java.util.List;

import solar.api.Event;
import solar.api.AttributeRecord;
import solar.api.ParsingException;


/**
 *  An event with plain encoding of square brackets.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PlainEvent.java,v 1.7 2003/10/05 15:24:39 glchen Exp $
 */
public class PlainEvent
    extends PlainAttributeRecord
    implements Event
{
    //=======================================================================
    // Constructors.
    //=======================================================================

    /** Constructs an empty plain event. */
    protected PlainEvent()
    {
        super();
    }

    /** Constructs an event by parsing an encoded string. */
    public PlainEvent(String rec)
        throws ParsingException
    {
        super(rec);
    }
    

    /** Constructs an event with a list of attributes. */
    public PlainEvent(List aList)
    {
        super(aList);
    }


    /** Constructs an event with a list of attributes. */
    public PlainEvent(AttributeRecord ar)
    {
        super(ar);
    }


    /** Constructs an event by copying from another one. */
    public PlainEvent(Event aEvent)
    {
        super((AttributeRecord)aEvent);
    }


    //=======================================================================
    // Implementation of solar.api.Adu interface.
    //=======================================================================

    public byte[] toBytes()
    {
        byte[] cnt = super.toString().getBytes();
        byte[] ret = new byte[cnt.length + 1];
        ret[0] = EVENT_PLAIN;
        System.arraycopy(cnt, 0, ret, 1, cnt.length);
        return ret;
    }

    
    //=======================================================================
    // Implementation of solar.api.Event interface.
    //=======================================================================

    public int getEncoding()
    {
        return EVENT_PLAIN;
    }
}
