/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.module.cache;


/**
 *  A container class for the object in the cache.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: CachedObject.java,v 1.1 2003/10/04 01:49:26 glchen Exp $
 */
public class CachedObject 
    implements CacheConstants
{
    /** The id for cached object. */
    protected Object _id;
    
    /** The actual cached object. */
    protected Object _obj;

    /** When created. **/
    protected long _created = 0;

    /** When it expires. **/
    private long _expires = 0;

    /** Is this object stale/expired? */
    private boolean _stale = false;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public CachedObject(Object id, Object obj)
    {
        _id = id;
        _obj = obj;
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Return the id of the cached object. */
    public Object getObjectId()
    {
        return _id;
    }
    
    /** Return the cached object. */
    public Object getObject()
    {
        return _obj;
    }

    /** Set the cached object. */
    public void setObject(Object obj)
    {
        _obj = obj;
    }

    /** Returns the creation time for the object. */
    public long getCreated()
    {
        return _created;
    }

    /** Set the creation time. */
    public void setCreated()
    {
        _created = System.currentTimeMillis();
    }

    /** Returns the expiration time for the object. */
    public long getExpires()
    {
        return _expires;
    }

    /** Set the expiration interval for the object. */
    public void setExpires(long expires)
    {
        _expires = expires;

        if (_expires == CACHE_TTL_FOREVER)
        {
            _stale = false;
        }
        else
        {
            _stale = (System.currentTimeMillis() - _created > _expires);
        }
    }

    /** Is the object stale? */
    public synchronized boolean isStale()
    {
        if (_expires == CACHE_TTL_FOREVER)
        {
            return false;
        }

        _stale = (System.currentTimeMillis() - _created > _expires);
        return _stale;
    }
}
