/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.module.cache;


/**
 *  Objects in a refreshable cache can be refreshed by adding them again.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RefreshableCache.java,v 1.1 2003/10/04 01:49:27 glchen Exp $
 */
public class RefreshableCache
    extends BaseCache
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public RefreshableCache()
    {
        super();
    }

    public RefreshableCache(int capacity, long ttl)
    {
        super(capacity, ttl);
    }

    
    //=======================================================================
    // Implementation of solar.service.cache.Cache interface.
    //=======================================================================

    /**
     *  Note if objectId exists in the cache, the previous cached object
     *  will be discarded and replaced by this one if they are different.
     */
    public synchronized void addObject(Object objectId, Object obj)
        throws CacheException
    {
        CachedObject co = new CachedObject(objectId, obj);
        
        if (_cache.get(objectId) == null)
        {
            if ((_capacity != CACHE_CAPACITY_INFINIT) && (_size >= _capacity))
                throw new CacheException("Cannot add more objects to cache.");
            
            _size ++;
        }

        // refresh the object if it exists already.
        co.setCreated();
        co.setExpires(_ttl);
        _cache.put(objectId, co);
    }
}
