/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service;


import solar.api.EventHandler;
import solar.api.RequestHandler;
import solar.service.dispatch.DispatchUnit;


/**
 *  Service is an independent module that has a name and a set of 
 *  configuration options.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Service.java,v 1.15 2003/09/25 05:29:55 glchen Exp $
 */
public interface Service
    extends EventHandler, RequestHandler
{
    /** Handle the data unit delivered by dispatch service. */
    public void handleDispatchUnit(DispatchUnit dunit)
        throws Exception;
    
    /** Set the service manager for this service. */
    public void setServiceManager(ServiceManager manager);


    /** Get the service manager for this service. */
    public ServiceManager getServiceManager();


    /** Set the name of this service. */
    public void setName(String name);


    /** Return the name of this service. */
    public String getName();


    /** Initiate the service, do not try to init dependent services here. */
    public void init()
        throws ServiceException;


    /** Initialize dependent services here. */
    public void initDependents()
        throws ServiceException;


    /** Return whether the service has been initialized. */
    public boolean hasInited();


    /** Start the service. */
    public void start();


    /** Shutdown the service. */
    public void stop();
}
