/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.dan;


import java.util.Hashtable;

import solar.api.Guid;
import solar.api.Address;
import solar.service.dan.lang.GraphSpec;


/**
 *  A graph manager maintains individual DAN graph that is potentially 
 *  distributed across several Planets.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: GraphManager.java,v 1.4 2003/10/05 18:50:55 glchen Exp $
 */
public abstract class GraphManager 
{
    /** The graph spec to be mantained. */    
    protected GraphSpec _gs;

    /** The table of (node_guid, current_address). */
    protected Hashtable _nodes;

    /** The table of (op_name, node_guid). */
    protected Hashtable _optable;

    /** The DAN service to which this graph manager is affiliated. */
    protected DanService _dan;
    
    /** The node manager holding the table (node_guid, NodeHandler). */
    protected NodeManager _nodeman;

    
    //=======================================================================
    // Constructors.
    //=======================================================================

    /** Constructs the manager with its reponsible graph spec. */
    public GraphManager(GraphSpec gs)
    {
        _gs = gs;
        _nodes = new Hashtable();
        _optable = new Hashtable();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Init the graph manager. */
    public void init(DanService dan)
    {
        _dan = dan;
        _nodeman = dan.getNodeManager();
    }

    
    /** Return the guid for loaded operator defined in graph spec. */
    public Guid getGuid(String opname)
    {
        return (Guid) _optable.get(opname);
    }

    
    /** Return the currrent address of the node with specified guid. */
    public Address getAddress(Guid guid)
    {
        return (Address) _nodes.get(guid);
    }


    /** Load the root operator and all its children as defined in the graph. */
    public abstract Guid loadOperator(String opname)
        throws DanException;
}
