/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.dan.lang;


import serp.util.Options;
import java.io.Serializable;
import java.util.List;
import java.util.Vector;
import java.util.Properties;

import solar.service.directory.Name;
import solar.SolarConstants;


/**
 *  The spec for a filter node as either source or sink of a connecting pipe.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: FilterSpec.java,v 1.1 2003/09/25 15:38:52 glchen Exp $
 */
public class FilterSpec
    implements Serializable, SolarConstants
{
    private static final long serialVersionUID = FILTER_SPEC_SVID;
    
    protected String _name;
    protected String _class;
    protected String _codebase;
    protected String _address;
    protected Properties _arguments;
    
    protected List _inputs;     // name of the pipes
    protected List _inports;
    protected List _outputs;    // name of the pipes
    protected List _outports;    
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public FilterSpec()
    {
        _arguments = new Properties();
        _inputs = new Vector();
        _inports = new Vector();
        _outputs = new Vector();
        _outports = new Vector();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void setName(String name)
    {
        _name = name;
    }

    public String getName()
    {
        return _name;
    }

    public void setClassname(String clazz)
    {
        _class = clazz;
    }

    public String getClassname()
    {
        return _class;
    }

    public void setCodebase(String codebase)
    {
        _codebase = codebase;
    }

    public String getCodebase()
    {
        return _codebase;
    }

    public void setAddress(String address)
    {
        _address = address;
    }

    public String getAddress()
    {
        return _address;
    }

    public void addArgument(String name, String value)
    {
        _arguments.setProperty(name, value);
    }

    public Options getArguments()
    {
        return new Options(_arguments);
    }

    //=======================================================================
    // setters and getters for inputs and input ports
    //=======================================================================
    
    public void addInput(String name)
    {
        _inputs.add(name);
    }

    public List getInputs()
    {
        return _inputs;
    }

    public void addInputPort(String name)
    {
        _inports.add(name);
    }

    public List getInputPorts()
    {
        return _inports;
    }

    //=======================================================================
    // setters and getters for outputs and output ports
    //=======================================================================

    public void addOutput(String name)
    {
        _outputs.add(name);
    }

    public List getOutputs()
    {
        return _outputs;
    }

    public void addOutputPort(String name)
    {
        _outports.add(name);
    }
    
    public List getOutputPorts()
    {
        return _outports;
    }

}
