/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.dan.lang;


import java.io.Serializable;
import java.util.Iterator;
import java.util.Hashtable;

import solar.api.ParsingException;
import solar.service.directory.Name;
import solar.SolarConstants;


/**
 *  The parsed specification of operator graph.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: GraphSpec.java,v 1.1 2003/09/25 15:38:52 glchen Exp $
 */
public class GraphSpec 
    implements Serializable, SolarConstants
{
    private static final long serialVersionUID = GRAPH_SPEC_SVID;
    
    protected Hashtable _graph;

    
    //=======================================================================
    // Constructors.
    //=======================================================================

    public GraphSpec()
    {
        _graph = new Hashtable();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public FilterSpec getFilter(String name)
    {
        return (FilterSpec) _graph.get(name);
    }


    public void putFilter(String name, FilterSpec filter)
    {
        _graph.put(name, filter);
    }


    public PipeSpec getPipe(String name)
    {
        return (PipeSpec) _graph.get(name);
    }

    public void putPipe(String name, PipeSpec pipe)
    {
        _graph.put(name, pipe);
    }

    /** Return the operator variable used by specified CS name. */
    public static String getOpVariable(Name name)
        throws ParsingException
    {
        String root = null;
        Iterator itr = name.getStrands().iterator();
        // get all the op variables used in the CS name
        // currently, we only allow one variable in one name but 
        // multiple usage is possible.
        while (itr.hasNext())
        {
            String strand = (String) itr.next();
            int p1 = strand.indexOf("$");
            int p2 = strand.indexOf(":");
            if (p1 != -1)
            {
                String r = strand.substring(p1+1, p2);
                if (root != null && !root.equals(r))
                {
                    throw new ParsingException(
                        "Only one operator allowed in dynamic names.");
                }
                root = r;
            }
        }
        return root;
    }
}
