/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.directory;


import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import solar.api.Attribute;
import solar.api.AttributeRecord;
import solar.api.ParsingException;
import solar.impl.PlainAttributeRecord;


/**
 *  A name with strings encoded in square brackets.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PlainName.java,v 1.2 2003/08/30 15:08:50 glchen Exp $
 */
public class PlainName 
    extends PlainAttributeRecord
    implements Name
{
    //=======================================================================
    // Constructors.
    //=======================================================================

    /** Constructs a name by parsing an encoded string. */
    public PlainName(String rec)
        throws ParsingException
    {
        super(rec);
    }

    /** Constructs a name with a list of attributes. */
    public PlainName(List aList)
    {
        super(aList);
    }

    /** Constructs a name with a list of attributes. */
    public PlainName(AttributeRecord ar)
    {
        super(ar);
    }

    /** Constructs a name by copying from another one. */
    public PlainName(Name aName)
    {
        super((AttributeRecord)aName);
    }

    
    //=======================================================================
    // Implementation of solar.api.Name interface.
    //=======================================================================

    /** Return the list of all strands. */
    public List getStrands()
    {
        List ret = new Vector();
        fillStrandsFromAR(this, null, ret);
        return ret;
    }


    //=======================================================================
    // Private helper methods.
    //=======================================================================

    private void fillStrandsFromAR(AttributeRecord ar, String prefix, List ret)
    {
        // a strand has the format of a=b=c=d=e, representing the path
        // from the root to a leaf node.
        Iterator itr = ar.getAttributes().iterator();
        while (itr.hasNext())
        {
            Attribute attr = (Attribute) itr.next();
            String key = attr.getKey();
            Object val = attr.getValue();
            String nextprefix = (prefix == null) ? key : prefix + "=" + key;
            if (val instanceof AttributeRecord)
            {
                // recursively fill the strands from children nodes
                fillStrandsFromAR((AttributeRecord)val, nextprefix, ret);
            }
            else
            {
                ret.add(nextprefix + "=" + val);
            }
        }
    }
}
