/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.directory;


import org.apache.log4j.Logger;
import java.io.Serializable;

import solar.api.Id;
import solar.api.Request;
import solar.api.Address;
import solar.service.directory.comm.*;
import solar.service.rpc.RpcService;
import solar.service.rpc.RpcConstants;
import solar.service.rpc.RpcException;


/**
 *  A proxy class for remote name observer. 
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RemoteDirectoryListener.java,v 1.2 2003/10/05 18:50:56 glchen Exp $
 */
class RemoteDirectoryListener 
    implements DirectoryListener, 
        Serializable,
        RpcConstants
{
    protected transient Logger _log;
    protected transient String _sname;
    protected transient RpcService _rpc;
    
    protected Id _id;
    protected Address _address;


    //===================================================================
    // Constructors.
    //===================================================================
    
    public RemoteDirectoryListener(Id id, Address address)
    {
        _id = id;
        _address = address;
    }


    //===================================================================
    // Public interface methods.
    //===================================================================

    public void setLogger(Logger log)
    {
        _log = log;
    }

    public void setRpcService(String sname, RpcService rpc)
    {
        _sname = sname;
        _rpc = rpc;
    }

    public Address getAddress()
    {
        return _address;
    }

    public Id getId()
    {
        return _id;
    }


    //===================================================================
    // Implementation of solar.service.directory.DirectoryObserver interface.
    //===================================================================

    public void nameAdded(Name name, NameRecord rec)
    {
        Request req = new NameAddedRequest(_id, name, rec);
        try
        {
            callRpc(req);
        }
        catch (RpcException re)
        {
            re.printStackTrace();
            _log.error(re);
        }
    }


    public void nameRemoved(Name name, NameRecord rec)
    {
        Request req = new NameRemovedRequest(_id, name, rec);
        try
        {
            callRpc(req);
        }
        catch (RpcException re)
        {
            re.printStackTrace();
            _log.error(re);
        }
    }


    public void nameUpdated(Name oldname, 
        NameRecord oldrec, Name newname, NameRecord newrec)
    {
        Request req = new NameUpdatedRequest(_id, 
            oldname, oldrec, newname, newrec);
        try
        {
            callRpc(req);
        }
        catch (RpcException re)
        {
            re.printStackTrace();
            _log.error(re);
        }
    }

    public void nameMigrated(Name name, 
        NameRecord oldrec, NameRecord newrec)
    {
        Request req = new NameMigratedRequest(_id, name, oldrec, newrec);
        try
        {
            callRpc(req);
        }
        catch (RpcException re)
        {
            re.printStackTrace();
            _log.error(re);
        }
    }


    //===================================================================
    // Private helper methods.
    //===================================================================

    private Object callRpc(Request req)
        throws RpcException
    {
        return _rpc.invoke(_address, _sname, req, RPC_TIMEOUT);
    }
}
