/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fap;


import solar.api.Id;
import solar.api.Address;
import solar.service.Service;
import solar.service.dispatch.DispatchService;
import solar.service.rpc.RpcService;
import solar.service.transport.TransportService;

/**
 *  A distributed service implementing Filter-and-Pipe (FAP) architectural
 *  pattern.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: FapService.java,v 1.12 2003/10/05 18:50:57 glchen Exp $
 */
public interface FapService
    extends Service
{
    /** Setup a local pipe and return the pipe ID. The parameter pid
     *  can be null and the method will return a generated ID for the pipe.*/
    public Id pconnect(Id pid, int type, 
        DataSource source, OutputPort outport,
        DataSink sink, InputPort inport)
        throws FapException;


    /** Setup a pipe that has a remote source. The parameter pid
     *  can be null and the method will return a generated ID for the pipe.*/
    public Id pconnect(Id pid, int type,
        Address raddr, Id rpid, OutputPort outport,
        DataSink sink, InputPort inport, boolean client)
        throws FapException;


    /** Setup a pipe that has a remote sink. The parameter pid
     *  can be null and the method will return a generated ID for the pipe.*/
    public Id pconnect(Id pid, int type,
        DataSource source, OutputPort outport,
        Address raddr, Id rpid, InputPort inport, boolean client)
        throws FapException;


    /** Return a pipe given its ID. */
    public Pipe getPipe(Id pid);
    

    /** Return the dispatch service. */
    public DispatchService getDispatchService();
    
    
    /** Return the RPC service used by a particular pipe. */
    public RpcService getRpcService(Id pid);


    /** Return the transport service used by a particular pipe. */
    public TransportService getTransportService(Id pid);

}
