/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fap;


import org.apache.log4j.Logger;

import solar.api.Id;
import solar.api.Event;
import solar.api.Request;
import solar.api.Address;
import solar.service.BaseService;
import solar.service.rpc.RpcService;
import solar.service.transport.TransportService;
import solar.service.ServiceException;


/**
 *  Implementation of Filter-and-Pipe (FAP) service.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SolarFapService.java,v 1.13 2003/10/05 18:50:57 glchen Exp $
 */
public class SolarFapService 
    extends FapClientService
{
    protected Logger _log = Logger.getLogger(SolarFapService.class);
    
    protected RpcService _coreRpc;
    protected RpcService _edgeRpc;

    protected TransportService _coreTransport;
    protected TransportService _edgeTransport;

    // a list of pipes that have either the source or sink on some Solar
    // clients, not on Planets. The reason for the different treatment
    // is that the routing/transport between client-planet and planet-planet
    // may be configurated differently
//    protected List _clients = new Vector();

    
    //=======================================================================
    //  Implementation of solar.service.Service interface.
    //=======================================================================
    
    public void initDependents()
        throws ServiceException
    {
        super.initDependents();     // must call at the top

        _edgeRpc = (RpcService) 
            getDependentService("edgeRpc", "edge_rpc");
        _edgeTransport = (TransportService) 
            getDependentService("edgeTransport", "edge_transport");
        
        _coreRpc = (RpcService) 
            getDependentService("coreRpc", "core_rpc");
        _coreTransport = (TransportService) 
            getDependentService("coreTransport", "core_transport");
    }

    
    //=======================================================================
    // Implementation of solar.service.fap.FapService interface.
    //=======================================================================
    
    public RpcService getRpcService(Id pid)
    {
        // if the pipe spans to Solar clients, return edge (client-planet)
        // RPC service, otherwise return core (planet-planet) RPC service
        if (_clients.contains(pid))
            return _edgeRpc;
        else
            return _coreRpc;
    }

    public TransportService getTransportService(Id pid)
    {
        // if the pipe spans to Solar clients, return edge (client-planet)
        // transport service, otherwise return core (planet-planet) transport
        // service
        if (_clients.contains(pid))
            return _edgeTransport;
        else
            return _coreTransport;
    }

    /*
    //=======================================================================
    // Private helper methods.
    //=======================================================================

    protected synchronized String makePipeId()
    {
        // Use the core transport to generate IDs on this Planet
        return _coreTransport.makeGuid();
    }
    */
}
