/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fap.comm;


import java.util.Vector;

import solar.api.Id;
import solar.api.Event;
import solar.api.Attribute;
import solar.impl.SerializedEvent;
import solar.service.fap.FapConstants;


/**
 *  A wrapper event encoding the data pushed down the pipe.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PipePushEvent.java,v 1.3 2003/09/26 19:47:14 glchen Exp $
 */
public class PipePushEvent 
    extends SerializedEvent
    implements FapConstants
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public PipePushEvent(Id pid, Object data)
    {
        super(new Vector());
        _attributes.add(new Attribute(FAP_PID, pid));
        _attributes.add(new Attribute(FAP_DATA, data));
    }

    public PipePushEvent(Event evt)
    {
        super(new Vector());
        _attributes.add(new Attribute(FAP_PID, evt.getValue(FAP_PID)));
        _attributes.add(new Attribute(FAP_DATA, evt.getValue(FAP_DATA)));
    }

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public Id getPipeId()
    {
        return (Id) getValue(FAP_PID);
    }

    public Object getData()
    {
        return getValue(FAP_DATA);
    }

    public static boolean isPipePushEvent(Event evt)
    {
        return evt.getValue(FAP_DATA) != null;
    }
}
