/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fork.client;


import serp.util.Options;
import org.apache.log4j.Logger;

import solar.service.SolarServices;
import solar.service.ServiceManager;
import solar.service.dispatch.DispatchService;
import solar.service.fork.ForkService;
import solar.Planet;


/**
 *  The interface to interact with Fork service.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: ForkClient.java,v 1.1 2003/10/08 21:55:23 glchen Exp $
 */
public class ForkClient 
{
    protected static Logger _log = Logger.getLogger("solar");
    
    private ForkService _fork;
    private ServiceManager _servman;
    private DispatchService _dispatch;
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public ForkClient(Options opts)
        throws Exception
    {
        String debug = opts.getProperty("debug");
        String layout = opts.getProperty("layout");
        String hostname = opts.getProperty("service.transport.hostname");
        String port = opts.getProperty("service.transport.port");
        String serialize = opts.getProperty("serialize");
        if (serialize == null)
        {
            serialize = opts.getProperty("service.transport.serialize");
        }
        if (opts.getProperty("cfg") == null)
        {
            opts = getDefaultOptions();
        }
        opts.setProperty("debug", debug);
        opts.setProperty("layout", layout);
        if (port != null)
            opts.setProperty("service.transport.port", port);
        if (hostname != null)
            opts.setProperty("service.transport.hostname", hostname);
        opts.setProperty("service.transport.serialize", serialize);
        construct(opts);
    }

    private Options getDefaultOptions()
    {
        Options opts = new Options();
        opts.setProperty("service.dispatch.classname", 
            "solar.service.dispatch.SolarDispatchService");

        opts.setProperty("service.fork.classname",
            "solar.service.fork.SolarForkService");

        opts.setProperty("service.rpc.classname",
            "solar.service.rpc.SolarRpcService");
            
        opts.setProperty("service.transport.classname", 
            "solar.service.transport.UdpTransportService");
        opts.setProperty("service.transport.port", 0);
        return opts;
    }

    private void construct(Options opts)
        throws Exception
    {
        // set the logging verbose level
        Planet.configLogger(_log, opts);

        // init service manager and relavant services
        _servman = SolarServices.getInstance();
        _servman.init(opts);
        _dispatch = (DispatchService) _servman.getService("dispatch");
        _fork = (ForkService) _servman.getService("fork");
    }

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void batchrun(String taskfile)
        throws Exception
    {
        _fork.batchrun(taskfile);
    }

    //=======================================================================
    // Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        ForkClient client = new ForkClient(opts);
        String taskfile = opts.getProperty("task", "task.cfg");
        client.batchrun(taskfile);
    }
}
