/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.pack;


import solar.api.Id;
import solar.api.Guid;
import solar.api.Event;
import solar.api.Address;
import solar.service.Service;
import solar.service.pubsub.Publisher;
import solar.service.pubsub.Subscriber;


/**
 *  PACK service allows application to customize the flow-control policies.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PackService.java,v 1.14 2003/10/11 02:49:31 glchen Exp $
 */
public interface PackService 
    extends Service
{
    /** Attach a PACK publisher to the service. */
    public void attachPublisher(Id pubId, Publisher publisher)
        throws PackException;


    /** Detach a PACK publisher from the service. */
    public void detachPublisher(Id pubId)
        throws PackException;


    /** Attach a PACK subscriber to the service. */
    public void attachSubscriber(Id subId, Subscriber subscriber)
        throws PackException;


    /** Detach a PACK subscriber from the service. */
    public void detachSubscriber(Id subId)
        throws PackException;

    
    /** Make a subscription to a PACK publisher. */
    public void subscribe(Id pubId, Id subId, Object policy)
        throws PackException;


    /** Publish an event to all subscribers. */
    public void publish(Id pubId, Event evt)
        throws PackException;


    /** Return a global unique ID. */
    public Guid makeGuid(Id id);
}
