/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import rice.pastry.Id;
import rice.pastry.PastryNode;

import solar.api.Adu;
import solar.api.Address;
import solar.impl.ByteArrayAdu;


/**
 *  A Pastry transport layer using object serialization.
 *  
 *  @author     Guanling chen
 *  @version    $Id: PastryObjectTransport.java,v 1.3 2003/10/05 18:51:03 glchen Exp $
 */
public class PastryObjectTransport 
    extends PastryTransport
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public PastryObjectTransport(PastryNode pn)
    {
        super(pn);

    }


    //=======================================================================
    // Implementation of solar.service.transport.Transport service.
    //=======================================================================

    public void send(byte[] data, Address dest)
        throws TransportException
    {
        send(data, dest, null);
    }

    public void send(byte[] data, Address dest, solar.api.Id multiplex)
        throws TransportException
    {
        send(new ByteArrayAdu(data), dest, multiplex);
    }

    public void send(Adu adu, Address dest)
        throws TransportException
    {
        send(adu, dest, null);
    }

    public void send(Adu adu, Address dest, solar.api.Id multiplex)
        throws TransportException
    {
        Id destNodeId = ((PastryAddress)dest).getNodeId();
        PastryMessage msg = new PastryMessage(adu, _maddr);

        Header header = new Header(multiplex);
        header.setSource(_laddr);
        header.setDestination(dest);
        msg.setHeader(header);
        
    	route(destNodeId, msg, null);
    }

    //=======================================================================
    //  Implementation of rice.pastry.client.CommonAPIAppl interface.
    //=======================================================================

    public void deliver(Id key, rice.pastry.messaging.Message msg)
    {
        PastryMessage p = (PastryMessage) msg;
        Header header = p.getHeader();
        Adu adu = p.getAdu();
        notifyDataReceived(adu, header);
    }
}
