/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import org.apache.log4j.Logger;
import java.net.Socket;

import solar.api.Id;
import solar.api.Adu;
import solar.api.Address;
import solar.impl.ByteArrayAdu;


/**
 *  The TCP transport layer using object serialization.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: TcpObjectTransport.java,v 1.5 2003/10/12 03:56:35 glchen Exp $
 */
public class TcpObjectTransport 
    extends TcpTransport
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public TcpObjectTransport()
        throws Exception
    {
        this(0);
    }
    
    public TcpObjectTransport(int port)
        throws Exception
    {
        super(port);
        _log = Logger.getLogger(TcpObjectTransport.class);
    }

    public TcpObjectTransport(String hostname, int port)
        throws Exception
    {
        super(hostname, port);
        _log = Logger.getLogger(TcpObjectTransport.class);
    }

    protected TcpConnection makeTcpConnection(Socket socket, SocketAddress peer)
        throws Exception
    {
        return new TcpConnection(true, socket, peer);
    }
    
    //=======================================================================
    // Implementation of ObjectTransport interface.
    //=======================================================================

    public void send(byte[] data, Address dest)
        throws TransportException
    {
        send(data, dest, null);
    }

    public void send(byte[] data, Address dest, Id multiplex)
        throws TransportException
    {
        send(new ByteArrayAdu(data), dest, multiplex);
    }
    
    public void send(Adu adu, Address dest)
        throws TransportException
    {
        send(adu, dest, null);
    }

    public void send(Adu adu, Address dest, Id multiplex)
        throws TransportException
    {
        SocketAddress rdest = (SocketAddress) dest;
        TcpConnection conn = getConnection(rdest);
        try
        {
            if (conn == null)
            {
                conn = openConnection(rdest);
            }
            Header header = new Header(multiplex);
            header.setSource(_laddr);
            header.setDestination(dest);
            conn.send(adu, header);
        }
        catch (Exception ex)
        {
            throw new TransportException("Error sending TCP data.", ex);
        }
    }
}
