/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.test;

import serp.util.Options;
import java.io.FileInputStream;

import solar.Client;
import solar.api.Event;
import solar.service.dan.Application;
import solar.util.FileUtil;


/**
 *  A simple Solar application that make subscriptions to print all
 *  receive events.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: EventPrinter.java,v 1.6 2003/09/26 19:47:19 glchen Exp $
 */
public class EventPrinter 
    extends Application
{
    //=======================================================================
    // Implementation of solar.service.dan.Application interface.
    //=======================================================================
    
    public void handleEvent(Event evt)
        throws Exception
    {
        System.out.println("Received: " + evt);
    }


    //=======================================================================
    // Main entrance.
    //=======================================================================

    /**
     *  Start the event printer and make subscription. Command line
     *  usage:
     *  <p>
     *  -host tahoe -port 5470 -query "[source=clock]"
     *  <p>or:<p>
     *  -host tahoe -port 5470 -subfile graph.xml -root \$root
     */
    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);
        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
        {
            opts.load(new FileInputStream(cfg));
            opts.setFromCmdLine(args);  // cmd opts overwrite opts in file
        }

        // get command line options
        String host = opts.getProperty("host", "localhost");
        int port = opts.getIntProperty("port", 5470);
        // create client facade object
        Client client = new Client(host, port, opts);

        String subfile = opts.getProperty("subfile");
        if (subfile == null)
        {
            String query = opts.getProperty("query", "[source=clock]");
            client.subscribe(query, new EventPrinter());
        }
        else
        {
            String root = opts.getProperty("root", "$root");
            client.subscribe(FileUtil.getContent(subfile), root, new EventPrinter());
        }
    }
}
