/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package solar.test;


import serp.util.Options;
import java.io.FileInputStream;
import java.net.InetAddress;

import solar.api.Id;
import solar.api.Event;
import solar.api.Address;
import solar.service.ServiceManager;
import solar.service.pack.PackPolicy;
import solar.service.pack.client.PackClient;
import solar.service.pack.lang.FilterConfig;
import solar.service.pubsub.Subscriber;


/**
 *  A standalone event subscriber using PACK service.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PackReceiver.java,v 1.26 2003/10/13 19:34:26 glchen Exp $
 */
public class PackReceiver 
{
    class Profiler implements Subscriber
    {
        int _scount = -1; // starting counter
        int _tcount = 0; // total received counter
        int _tlatency = 0; // total latency
        boolean _summary = true;
        public Profiler(boolean summary)
        {
            _summary = summary;
        }
        public void handleEvent(Event evt)
            throws Exception
        {
            long tstamp = Long.parseLong((String)evt.getValue("tstamp"));
            long latency = System.currentTimeMillis() - tstamp;
            int counter = Integer.parseInt((String) evt.getValue("counter"));
            if (_summary) {
                if (_scount == -1) {
                    _scount = counter;
                }
                _tcount ++;
                _tlatency += latency;
                if (_tcount % 100 == 0) {
                    int total = counter - _scount + 1;
                    double lossrate = (total - _tcount) / (total * 1.0);
                    System.out.println((total-_tcount) 
                        + " " + lossrate + " " + _tlatency/(_tcount*1.0));
                }
            }
            else {
                System.out.println("counter " + counter + " latency " + latency);
            }
        }
    }
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public PackReceiver()
    {
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    public void run(PackClient client, Options opts)
        throws Exception
    {
        boolean summary = opts.getBooleanProperty("summary", true);
        Profiler profiler = new Profiler(summary);
        Id pubid = new Id(opts.getProperty("pubid", "pack-pub"));
        Id subid = new Id(opts.getProperty("subid", "pack-sub"));

        client.attachSubscriber(subid, profiler);
/*        String policy = "<pack>"
            + "<filter name=\"ODD\">" 
            + "<argument name=\"attribute\" value=\"counter\"/>"
            + "</filter>"
            + "<filter name=\"EVEN\">" 
            + "<argument name=\"attribute\" value=\"counter\"/>"
            + "</filter>"
            + "</pack>";*/
        client.subscribe(pubid, subid, getPolicy());
        /*
        Thread.sleep(5000);
        client.detachSubscriber(subid);
        System.out.println("detached...");
        Thread.sleep(10000);
        System.out.println("attached...");
        client.attachSubscriber(subid, profiler);
        client.subscribe(pubid, subid, policy);*/
    }

    private PackPolicy getPolicy()
    {
        FilterConfig fc = new FilterConfig();
        fc.setName("RANDOM");
        fc.addArgument("percent", "5");
        PackPolicy policy = new PackPolicy();
        policy.setCoolFilter(fc);
        policy.setWarmFilter(fc);
        policy.setHotFilter(fc);
        return policy;
    }

    //=======================================================================
    // Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
        {
            opts.load(new FileInputStream(cfg));
            opts.setFromCmdLine(args);
        }
        
        // get command line options
        String host = opts.getProperty("host", "localhost");
        int port = opts.getIntProperty("port", 5470);
        // create client facade object
        PackClient client = new PackClient(host, port, opts);
        new PackReceiver().run(client, opts);
    }
}
