/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus;


import java.util.Hashtable;

import solar.api.Event;
import solar.api.Request;
import solar.service.dan.Operator;


/**
 *  A location aggregator that tracks the latest location of every object.
 *  It pubishes a location-change event whenever an object moves to a
 *  different zone.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: VersusTracker.java,v 1.6 2003/09/25 05:30:04 glchen Exp $
 */
public class VersusTracker
    extends Operator
{
    private Hashtable _state = new Hashtable();


    //=======================================================================
    // Implementation of solar.service.dan.Operator interface.
    //=======================================================================
    
    public void handleEvent(Event evt)
        throws Exception
    {
        String name = (String) evt.getValue("name");
        String zone = (String) evt.getValue("zone");
        String oldzone = (String) _state.get(name);
        if (zone.indexOf("Unknown") != -1 || zone.indexOf("radio") != -1)
        {
            return;
        }
        if (oldzone == null || !oldzone.equals(zone))
        {
            _state.put(name, zone);
            publish(evt);
        }
    }

    public Object handleQuery(Request query)
        throws Exception
    {
//        String name = (String) query.getArgument("name");
//        return _state.get(name);
        return null;
    }
}
