/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
                                                                                                                                                             
package versus.meeting;


import java.io.*;
import java.util.logging.*;
import java.util.*;
import java.text.*;
import javax.mail.*;
import javax.mail.internet.*;


/**
 *  The recorder provides an user interface for user to record meeting
 *  starts and ends.  The produced log is used for training various
 *  learning algorithms for online meeting detection.
 *  
 *  @author     Guanling Chen
 *  @author     Jue Wang
 */
public class MeetingRecorder extends javax.swing.JDialog {
    
    private static Logger logger = Logger.getLogger(
        MeetingRecorder.class.getName(), 
        "versus.meeting.MeetingResource");
	
    private long startedAt;
    
    /** Creates new form MeetingController */
    public MeetingRecorder(java.awt.Frame parent, boolean modal) 
		throws Exception
	{
        super(parent, modal);
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        StartButton = new javax.swing.JButton();
        StopButton = new javax.swing.JButton();
        CancelButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();

        getContentPane().setLayout(null);

        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		int fwidth = (int) (screenSize.width * 0.8); // 394
		int fheight = (int) (screenSize.height * 0.8); // 291
        setBounds((screenSize.width-fwidth)/2, 
			(screenSize.height-fheight)/2, fwidth, fheight);

        int bwidth = fwidth / 3;
        int bheight = fheight / 3;
        int b1x = bwidth / 3;   // for start button
        int b1y = bheight / 3;
        int b2x = b1x*2 + bwidth;   // for stop button
        int b2y = b1y;
        int b3x = bwidth;     // for close button
        int b3y = b1y*2 + bheight;
        
        StartButton.setBackground(new java.awt.Color(0, 255, 0));
        StartButton.setFont(new java.awt.Font("Dialog", 1, 18));
        StartButton.setText("Start");
        StopButton.setEnabled(false);
        StartButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                StartButtonActionPerformed(evt);
            }
        });

        getContentPane().add(StartButton);
        StartButton.setBounds(b1x, b1y, bwidth, bheight);

        StopButton.setBackground(new java.awt.Color(102, 102, 102));
        StopButton.setFont(new java.awt.Font("Dialog", 1, 18));
        StopButton.setText("Stop");
        StopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                StopButtonActionPerformed(evt);
            }
        });

        getContentPane().add(StopButton);
        StopButton.setBounds(b2x, b2y, bwidth, bheight);

        CancelButton.setBackground(new java.awt.Color(0, 0, 255));
        CancelButton.setFont(new java.awt.Font("Dialog", 1, 18));
        CancelButton.setForeground(new java.awt.Color(255, 255, 255));
        CancelButton.setText("Close");
        CancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                CancelButtonActionPerformed(evt);
            }
        });

        getContentPane().add(CancelButton);
        CancelButton.setBounds(b3x, b3y, bwidth, bheight);

        jLabel1.setFont(new java.awt.Font("Dialog", 1, 24));
        jLabel1.setForeground(new java.awt.Color(0, 102, 102));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText("Meeting Recorder");
        getContentPane().add(jLabel1);
        jLabel1.setBounds(b3x-10, 5, bwidth+20, 40);


    }//GEN-END:initComponents
    
    private void CancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_CancelButtonActionPerformed
        setVisible(false);
        dispose();
    }//GEN-LAST:event_CancelButtonActionPerformed
    
    private void StopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_StopButtonActionPerformed
        StopButton.setEnabled(false);
        StopButton.setBackground(new java.awt.Color(102, 102, 102));
        StartButton.setEnabled(true);
        StartButton.setBackground(new java.awt.Color(0, 255, 0));
        long stoppedAt = System.currentTimeMillis();
        Date stopDate = new Date(stoppedAt);
        SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");
        String stopStr = sdf.format(stopDate);

		double duration = (stoppedAt - startedAt)/(1000*60.0);
		long dur = (long) ((duration*10) % 10);
		dur = (dur>5) ? (long) duration + 1 : (long) duration;

		String startStr = sdf.format(new Date(startedAt));
        logger.info("stop "+stopStr+" "+stoppedAt);
        logger.info("last "+ dur + " minutes");
        String msg = "start "+ startStr + " " + startedAt + "\n";
		msg += "stop " + stopStr + " " + stoppedAt + "\n";
		msg += "last " + dur + " minutes\n";
        sendEmail(msg);

//        logger.info("stop "+dateStr+" "+stoppedAt+" "+(stoppedAt - startedAt)/1000/60);
//        sendEmail("stop "+dateStr+" "+stoppedAt+" "+(stoppedAt - startedAt)/1000/60);
    }//GEN-LAST:event_StopButtonActionPerformed
    
    private void StartButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_StartButtonActionPerformed
        StartButton.setEnabled(false);
        StartButton.setBackground(new java.awt.Color(102, 102, 102));
        StopButton.setEnabled(true);
        StopButton.setBackground(new java.awt.Color(255, 0, 0));
        startedAt = System.currentTimeMillis();
        Date startDate = new Date(startedAt);
        SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");
        String dateStr = sdf.format(startDate);
        logger.info("start "+dateStr+" "+startedAt);
        sendEmail("start "+dateStr+" "+startedAt);
    }//GEN-LAST:event_StartButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    private static void sendEmail(java.lang.String message) {
        try {
            Properties props = System.getProperties();
            props.put("mail.smtp.host",logger.getResourceBundle().getString("mail.smtp.host"));
            Session session = Session.getDefaultInstance(props);
            Message msg = new MimeMessage(session);
            InternetAddress from = new InternetAddress(logger.getResourceBundle().getString("mail.from"));
            InternetAddress to[] = InternetAddress.parse(logger.getResourceBundle().getString("mail.to"));
            msg.setFrom(from);
            msg.setRecipients(Message.RecipientType.TO,to);
            msg.setSubject("Meeting Notification");
            msg.setSentDate(new Date());
            msg.setText(message);
            Transport.send(msg);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) 
		throws Exception
	{
        new MeetingRecorder(new javax.swing.JFrame(), true).show();
        System.exit(0);
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton CancelButton;
    private javax.swing.JButton StartButton;
    private javax.swing.JButton StopButton;
    private javax.swing.JLabel jLabel1;
    // End of variables declaration//GEN-END:variables
    
}

